/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : DaylyPanel
   date       : 03.01.98
   description: an panel that displays the day view of the calendar
   modified   : 19.02.98 - itemStateChanged calculates start and
                           end time of selection

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class DaylyPanel extends Panel
  implements Observer,ItemListener {

  // class variable declarations
  private final static int MINUTES_PER_CELL   = 30;
  private final static int TIME_WIDTH         = 40;
  private final static int ENTRY_WIDTH        = 270;
  private final static int[] A_COL_WIDTH      = {TIME_WIDTH,ENTRY_WIDTH};

  // instance variable declarations
  private LoginInfoModel lModel;
  private ScopeModel sModel;
  private DateListModel dModel;
  private int vSize;
  private SpreadSheetCanvas spreadSheet;
  private DateChooserPanel dateChooser;
  private Calendar calendar;
  private SimpleDateFormat formatter;
  // constructors
  public DaylyPanel(LoginInfoModel lModel, ScopeModel sModel,
                    DateListModel dModel, int vSize) {
    int i,n;

    formatter   = new SimpleDateFormat("H:mm",CalendarCreator.getLocale());
    this.lModel = lModel;
    this.sModel = sModel;
    this.dModel = dModel;
    this.sModel.addObserver(this);
    this.dModel.addObserver(this);
    this.vSize = vSize;

    calendar = CalendarCreator.get();

    ScrollPane sp = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
    sp.setSize(TIME_WIDTH+ENTRY_WIDTH+sp.getVScrollbarWidth(),vSize);
    spreadSheet = new SpreadSheetCanvas(24*60/MINUTES_PER_CELL,
                                        A_COL_WIDTH,
                                        SystemColor.control,true);
    sp.add(spreadSheet);
    calendar.setTime(sModel.getDate());
    n = 24*60/MINUTES_PER_CELL;
    for (i=0; i<n; i++) {
      spreadSheet.setCell(0,i,0,formatter.format(calendar.getTime()));
      calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
    } // end for
    add(sp);
    calendar.setTime(new Date());
    add(dateChooser = new DateChooserPanel(calendar,CalendarCreator.getLocale()));
    spreadSheet.addItemListener(this);
    dateChooser.addItemListener(this);
  } // end constructor DaylyPanel

  // methods
  public void update(Observable obs, Object arg) {
    if (sModel.getScopeWidth() != ScopeModel.DAYLY) {
      spreadSheet.setBufferEnabled(false);
      return;
    }
    else {
      spreadSheet.setBufferEnabled(true);
    } // end if

    if (obs.equals(sModel)) {
      if ((sModel.dateChanged()) &&
          (!dateChooser.equals(arg))) dateChooser.setDate(sModel.getDate());
    }
    else { // if observable equals dModel
      if (dModel.isNew()) {
        // update spreadsheet
        Enumeration enum = dModel.getCalendarEntries();
        CalendarEntry entry;
        Date oldDate;
        int n = 24*60/MINUTES_PER_CELL;
        int i,l;

        // delete spreadsheet
        spreadSheet.unSelect();
        for (i=0; i<n; i++) {
          spreadSheet.deleteCell(1,i);
        } // end for

        // set calendar to start
        calendar.setTime(sModel.getStartOfScope());
        oldDate = calendar.getTime();
        calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
        i = 0; l = 0;
        while (enum.hasMoreElements()) {
          entry = (CalendarEntry)enum.nextElement();
          if (entry.getStart().before(oldDate)) {
            Assert.out.println("DaylyPanel: illegal date in list");
            continue;
          } // end if
          while (!calendar.getTime().after(entry.getStart())) {
            oldDate = calendar.getTime();
            calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
            i++; l = 0;
          } // end while
          if (i >= n) {
            Assert.out.println("DaylyPanel: date after end of spreadsheet");
            break;
          } // end if
          spreadSheet.setCell(1,i,l,entry);
          l++;
          while ((entry.getEnd().after(calendar.getTime())) && (i < (n-1))) {
            oldDate = calendar.getTime();
            calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
            i++; l = 1;
            spreadSheet.setUplinkCell(1,i);
          } // end while
        } // end while
        spreadSheet.repaint();
      } // end if
    } // end if
  } // end update

  public void itemStateChanged(ItemEvent e) {
    if (e.getItemSelectable().equals(dateChooser)) {
      sModel.setDate(dateChooser.getDate(),dateChooser);
    }
    else { // if Itemselectable equals spreadSheet
      Object aObj[] = spreadSheet.getSelectedObjects();
      Point point   = spreadSheet.getSelectedIndex();

      if (point == null) {
        sModel.setSelectedObject(aObj[0]);
      }
      else if (point.x == 0) {
        spreadSheet.unSelect();
        sModel.setSelectedObject(aObj[0]);
      }
      else {
        calendar.setTime(sModel.getStartOfScope());
        calendar.add(Calendar.MINUTE,MINUTES_PER_CELL*point.y);
        Date start = new Date(calendar.getTime().getTime());
        calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
        Date end = new Date(calendar.getTime().getTime());
        sModel.setSelectedObject(aObj[0],start,end);
      } // end if
    } // end if
  } // end itemStateChanged

} // end DaylyPanel

/* ******************************************************************
   end of file
*********************************************************************/
