/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : EditDateDlg
   date       : 12.12.97
   description: a dialog to edit a complete date
   modified   : 13.03.98 - dialog is no longer initially visible
                19.03.98 - bug dealing around with CardLayout removed

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class EditDateDlg extends Dialog
  implements ActionListener, ItemListener {

  // class variable declarations

  // some constants for the mode constructor parameter
  public final static int NEW             = 0;
  public final static int EDIT            = 1;
  public final static int VIEW            = 2;
  public final static int VIEW_AND_DELETE = 3;
  public final static int FIND_RESULT_NEW = 4;

  private final static boolean DISPLAY_ALARM_SETTINGS = false;

  private final static String GENERAL_PANEL  = "g";
  private final static String PERSONS_PANEL  = "p";
  private final static String NOTES_PANEL    = "n";
  private final static String PERIODIC_PANEL = "per";

  // instance variable declarations
  private Panel cardPanel;
  private TabbedChoice tabbedChoice;
  private CardLayout cardLayout;
  private Frame parent;
  private CalendarEntry entry;
  private Vector personsVector;
  private int mode;
  private Button okBtn;
  private Button cancelBtn;
  private Button deleteBtn;
  private TextField dateText;
  private Button dateSelBtn;
  private TextField startText;
  private TextField endText;
  private TextField creatorText;
  private TextField descText;
  private TextField locText;
  private Checkbox alarmCheckbox;
  private TextField alarmText;
  private Label alarmLabel;
  private Choice alarmChoice;
  private Checkbox privateCheckbox;
  private List personsList;
  private TextArea notesText;
  private Checkbox periodicCheckbox;
  private CheckboxGroup cbg;
  private Checkbox daylyCheckbox;
  private Checkbox weeklyCheckbox;
  private Checkbox monthlyCheckbox;
  private TextField daylyText;
  private Label daylyLabel;
  private TextField weeklyText;
  private Label weeklyLabel;
  private TextField monthlyText;
  private Label monthlyLabel1;
  private Label monthlyLabel2;
  private Choice periodicChoice;
  private Checkbox expirCheckbox;
  private TextField expirText;
  private Button expirSelBtn;
  private SimpleDateFormat dateFormatter;
  private SimpleDateFormat timeFormatter;
  private SimpleDateFormat dateTimeFormatter;
  private boolean canceled    = false;
  private boolean autoCancel  = false;
  private boolean autoDisable = false;
  private boolean deleted     = false;

  // inner classes
  class QuitListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      canceled = true;
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        canceled = true;
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end QuitListener

  class DateListener extends WindowAdapter {

    private DateChooserDlg dateDlg;

    public DateListener(DateChooserDlg dateDlg) {
      super();
      this.dateDlg = dateDlg;
    } // end constructor DateListener

    public void windowClosed(WindowEvent e) {
      if (autoDisable) { autoCancel = true; autoDisable = false; }
      if (!dateDlg.getCanceled()) {
        if (tabbedChoice.getSelectedIndex() == 0)
          dateText.setText(dateFormatter.format(dateDlg.getDate()));
        else // if index is 3
          expirText.setText(dateFormatter.format(dateDlg.getDate()));
        // end if
      } // end if
    } // end windowClosed
  
  } // end DateListener

  class MsgListener extends WindowAdapter {

    public void windowClosed(WindowEvent e) {
      autoCancel  = true;
      autoDisable = false;
    } // end windowClosed
  
  } // end DateListener

  // constructors
  public EditDateDlg(Frame parent, CalendarEntry entry,
                     int mode, String[] persons) {
    super(parent);
    init(parent,entry,mode,persons);
  } // end constructor EditDateDlg

  public EditDateDlg(Frame parent, boolean modal, CalendarEntry entry,
                     int mode, String[] persons) {
    super(parent,modal);
    init(parent,entry,mode,persons);
  } // end constructor EditDateDlg

  public EditDateDlg(Frame parent, String title, CalendarEntry entry,
                     int mode, String[] persons) {
    super(parent,title);
    init(parent,entry,mode,persons);
  } // end constructor EditDateDlg

  public EditDateDlg(Frame parent, String title, boolean modal,
                     CalendarEntry entry, int mode, String[] persons) {
    super(parent,title,modal);
    init(parent,entry,mode,persons);
  } // end constructor EditDateDlg

  // methods
  private void init(Frame parent, CalendarEntry entry,
                    int mode, String[] persons) {
    int i;

    this.parent   = parent;
    this.entry    = entry;
    this.mode     = mode;
    personsVector = new Vector();
    for (i=0; i<persons.length; i++) {
      personsVector.addElement(persons[i]);
    } // end for

    addWindowListener(new QuitListener());

    dateFormatter     = new SimpleDateFormat("dd.MM.yy");
    timeFormatter     = new SimpleDateFormat("HH:mm");
    dateTimeFormatter = new SimpleDateFormat("dd.MM.yy.HH:mm");
    setLayout(new GridLayout(1,1));

    tabbedChoice = new TabbedChoice();
    Panel btnPanel = new Panel(new GridLayout(1,3,10,0));
    okBtn = new Button("OK");
    okBtn.addActionListener(this);
    btnPanel.add(okBtn);
    cancelBtn = new Button("Cancel");
    cancelBtn.addActionListener(this);
    btnPanel.add(cancelBtn);
    deleteBtn = new Button("Delete");
    if ((mode == NEW) || (mode == VIEW) || (mode == FIND_RESULT_NEW))
      deleteBtn.setEnabled(false);
    else
      deleteBtn.addActionListener(this);
    // end if
    btnPanel.add(deleteBtn);

    cardLayout = new CardLayout();
    cardPanel  = new Panel(cardLayout);

    // the controls for the general settings
    {
      GridBagLayout gridbag     = new GridBagLayout();
      GridBagConstraints constr = new GridBagConstraints();
      Panel generalPanel       = new Panel(gridbag);
      generalPanel.setBackground(SystemColor.control);

      // create the widgets
      Label dateLabel = new Label("Date:");
      dateText = new TextField(8);
      dateSelBtn = new Button("Select");
      dateSelBtn.addActionListener(this);
      Label startLabel = new Label("Start:");
      startText = new TextField(10);
      Label endLabel = new Label("End:");
      endText = new TextField(10);
      if (entry.getStart().getTime() != CalendarEntry.INVALID_DATE) {
        dateText.setText(dateFormatter.format(entry.getStart()));
        startText.setText(timeFormatter.format(entry.getStart()));
        if (entry.getEnd().getTime() != CalendarEntry.INVALID_DATE) {
          endText.setText(timeFormatter.format(entry.getEnd()));
        } // end if
      } // end if
      Label creatorLabel = new Label("Creator:");
      creatorText = new TextField(entry.getCreator());
      creatorText.setEditable(false);
      Panel p1 = new Panel(new GridLayout(2,1,0,10));
      Panel p2 = new Panel(new GridLayout(2,1,0,10));
      p1.add(new Label("Description:"));
      descText = new TextField(entry.getDescription(),30);
      p2.add(descText);
      p1.add(new Label("Location:"));
      locText = new TextField(entry.getLocation(),30);
      p2.add(locText);
      alarmCheckbox = new Checkbox("Alarm");
      alarmCheckbox.addItemListener(this);
      alarmText = new TextField(2);
      alarmChoice = new Choice();
      alarmChoice.add("Minute(s)");
      alarmChoice.add("Hour(s)");
      alarmLabel = new Label("before");
      if ((entry.getStart().getTime() != CalendarEntry.INVALID_DATE) &&
          (entry.getAlertTime().getTime() != CalendarEntry.INVALID_DATE)) {
        Date timeDiff = new Date(entry.getStart().getTime()-
                                 entry.getAlertTime().getTime());
        Calendar cal = CalendarCreator.get();
        cal.setTime(timeDiff);
        if (cal.get(Calendar.HOUR) == 0) {
          alarmText.setText(Integer.toString(cal.get(Calendar.MINUTE)));
        }
        else {
          alarmText.setText(Integer.toString(cal.get(Calendar.HOUR)));
          alarmChoice.select(1);
        } // end if
        alarmCheckbox.setState(true);
      }
      else {
        alarmText.setText("15");
      } // end if
      privateCheckbox = new Checkbox("Private");
      if (entry.isPrivate()) privateCheckbox.setState(true);
      updateEnabled(0);

      // do the layout
      constr.anchor     = GridBagConstraints.WEST;
      constr.insets     = new Insets(5,6,5,6);
      constr.gridwidth  = 1;
      constr.gridheight = 1; 
      gridbag.setConstraints(dateLabel,constr); 
      generalPanel.add(dateLabel);
      constr.gridwidth = GridBagConstraints.RELATIVE; 
      gridbag.setConstraints(dateText,constr); 
      generalPanel.add(dateText);
      constr.gridwidth = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(dateSelBtn,constr); 
      generalPanel.add(dateSelBtn);
      constr.gridwidth = 1; 
      gridbag.setConstraints(startLabel,constr); 
      generalPanel.add(startLabel);
      constr.gridwidth = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(startText,constr); 
      generalPanel.add(startText);
      constr.gridwidth = 1;
      gridbag.setConstraints(endLabel,constr); 
      generalPanel.add(endLabel);
      constr.gridwidth = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(endText,constr); 
      generalPanel.add(endText);
      constr.gridwidth = 1;
      gridbag.setConstraints(creatorLabel,constr); 
      generalPanel.add(creatorLabel);
      constr.gridwidth = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(creatorText,constr); 
      generalPanel.add(creatorText);
      if (!DISPLAY_ALARM_SETTINGS)
        constr.gridheight = GridBagConstraints.RELATIVE;
      // end if
      constr.gridwidth = 1;
      gridbag.setConstraints(p1,constr); 
      generalPanel.add(p1);
      constr.gridwidth = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(p2,constr); 
      generalPanel.add(p2);
      if (DISPLAY_ALARM_SETTINGS) {
        constr.gridwidth  = 1;
        constr.gridheight = GridBagConstraints.RELATIVE;
        gridbag.setConstraints(alarmCheckbox,constr); 
        generalPanel.add(alarmCheckbox);
        gridbag.setConstraints(alarmText,constr); 
        generalPanel.add(alarmText);
        constr.gridwidth = GridBagConstraints.RELATIVE; 
        gridbag.setConstraints(alarmChoice,constr); 
        generalPanel.add(alarmChoice);
        constr.gridwidth = GridBagConstraints.REMAINDER;
        gridbag.setConstraints(alarmLabel,constr); 
        generalPanel.add(alarmLabel);
      } // end if
      constr.gridwidth  = GridBagConstraints.REMAINDER;
      constr.gridheight = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(privateCheckbox,constr);
      generalPanel.add(privateCheckbox);
      tabbedChoice.add("General");
      cardPanel.add(GENERAL_PANEL,generalPanel);
    }

    // the controls for the persons settings
    {
      Panel personsPanel = new Panel();
      String creator      = entry.getCreator();
      int n               = personsVector.size();
      int j;

      personsPanel.setBackground(SystemColor.control);
      personsList = new List(10,true);
      j = 0;
      for (i=0; i<n; i++) {
        String person = (String)personsVector.elementAt(i);
        if (!creator.equals(person)) {
          personsList.add((String)personsVector.elementAt(i));
          if (entry.hasPerson(person)) {
            personsList.select(j);
          } // end if
          j++;
        } // end if
      } // end for
      updateEnabled(1);
      personsPanel.add(personsList);
      tabbedChoice.add("Persons");
      //tabbedChoice.select(0);
      cardPanel.add(PERSONS_PANEL,personsPanel);
    }

    // the controls for the notes editor
    {
      Panel notesPanel  = new Panel();

      notesPanel.setBackground(SystemColor.control);
      notesText = new TextArea(12,40);
      notesText.setText(entry.getNotes());
      updateEnabled(2);
      notesPanel.add(notesText);
      tabbedChoice.add("Notes");
      //tabbedChoice.select(0);
      cardPanel.add(NOTES_PANEL,notesPanel);
    }

    // the controls for the periodic settings
    {
      GridBagLayout gridbag     = new GridBagLayout();
      GridBagConstraints constr = new GridBagConstraints();
      Panel periodicPanel      = new Panel(gridbag);
      periodicPanel.setBackground(SystemColor.control);

      // create the widgets
      boolean isPeriodic   = entry.isPeriodic();
      int periodicType     = entry.getPeriodic();
      boolean hasExpirDate = entry.hasExpirDate();
      periodicCheckbox = new Checkbox("Periodic",isPeriodic);
      periodicCheckbox.addItemListener(this);
      cbg              = new CheckboxGroup();
      daylyCheckbox    = new Checkbox("Dayly  - every",cbg,false);
      daylyCheckbox.addItemListener(this);
      weeklyCheckbox   = new Checkbox("Weekly - every",cbg,false);
      weeklyCheckbox.addItemListener(this);
      monthlyCheckbox  = new Checkbox("Monthly",cbg,false);
      monthlyCheckbox.addItemListener(this);
      switch (periodicType) {
        case CalendarEntry.NOT_PERIODICAL:
        case CalendarEntry.EVERY_DAY:
          cbg.setSelectedCheckbox(daylyCheckbox);
        break;
        case CalendarEntry.EVERY_WEEK:
          cbg.setSelectedCheckbox(weeklyCheckbox);
        break;
        default:        
          cbg.setSelectedCheckbox(monthlyCheckbox);
      } // end switch
      if (periodicType != CalendarEntry.EVERY_DAY) daylyText = new TextField("1",2);
        else daylyText = new TextField(Integer.toString(entry.getDistance()),2);
      daylyLabel = new Label("Day(s)");
      if (periodicType != CalendarEntry.EVERY_WEEK) weeklyText = new TextField("1",2);
        else weeklyText = new TextField(Integer.toString(entry.getDistance()),2);
      weeklyLabel = new Label("Week(s)");
      if ((periodicType == CalendarEntry.EVERY_MONTH) ||
          (periodicType == CalendarEntry.EVERY_MONTH_EXACT) ||
          (periodicType == CalendarEntry.EVERY_MONTH_MATCH)) {
        monthlyText = new TextField(Integer.toString(entry.getDistance()),2);
       }
       else {
        monthlyText = new TextField("1",2);
      } // end if
      periodicChoice = new Choice();
      periodicChoice.add("Same Day");
      periodicChoice.add("Same Week of Month");
      if (periodicType == CalendarEntry.EVERY_MONTH_MATCH) periodicChoice.select(1);
      monthlyLabel1 = new Label("every");
      monthlyLabel2 = new Label("Month(s)");
      expirCheckbox = new Checkbox("expires at",hasExpirDate);
      expirCheckbox.addItemListener(this);
      expirText   = new TextField(8);
      if (entry.hasExpirDate()) {
        expirCheckbox.setState(true);
        expirText.setText(dateFormatter.format(entry.getExpirTime()));
      } // end if
      expirSelBtn = new Button("Select");
      expirSelBtn.addActionListener(this);
      updateEnabled(3);

      // do the layout
      constr.anchor     = GridBagConstraints.WEST;
      constr.insets     = new Insets(5,6,5,6);
      constr.gridwidth  = GridBagConstraints.REMAINDER; 
      constr.gridheight = 1; 
      gridbag.setConstraints(periodicCheckbox,constr); 
      periodicPanel.add(periodicCheckbox);
      constr.gridwidth  = 1; 
      gridbag.setConstraints(daylyCheckbox,constr); 
      periodicPanel.add(daylyCheckbox);
      constr.gridwidth  = GridBagConstraints.RELATIVE; 
      gridbag.setConstraints(daylyText,constr); 
      periodicPanel.add(daylyText);
      constr.gridwidth  = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(daylyLabel,constr); 
      periodicPanel.add(daylyLabel);
      constr.gridwidth  = 1; 
      gridbag.setConstraints(weeklyCheckbox,constr); 
      periodicPanel.add(weeklyCheckbox);
      constr.gridwidth  = GridBagConstraints.RELATIVE; 
      gridbag.setConstraints(weeklyText,constr); 
      periodicPanel.add(weeklyText);
      constr.gridwidth  = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(weeklyLabel,constr); 
      periodicPanel.add(weeklyLabel);

      constr.gridwidth  = 1;
      gridbag.setConstraints(monthlyCheckbox,constr);
      periodicPanel.add(monthlyCheckbox);
      constr.gridwidth  = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(periodicChoice,constr);
      periodicPanel.add(periodicChoice);

      constr.gridwidth  = 1;
      constr.gridheight = GridBagConstraints.RELATIVE;
      gridbag.setConstraints(monthlyLabel1,constr);
      periodicPanel.add(monthlyLabel1);
      constr.gridwidth  = GridBagConstraints.RELATIVE; 
      gridbag.setConstraints(monthlyText,constr); 
      periodicPanel.add(monthlyText);
      constr.gridwidth  = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(monthlyLabel2,constr); 
      periodicPanel.add(monthlyLabel2);

      constr.gridwidth  = 1;
      constr.gridheight = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(expirCheckbox,constr);
      periodicPanel.add(expirCheckbox);
      constr.gridwidth  = GridBagConstraints.RELATIVE; 
      gridbag.setConstraints(expirText,constr); 
      periodicPanel.add(expirText);
      constr.gridwidth  = GridBagConstraints.REMAINDER; 
      gridbag.setConstraints(expirSelBtn,constr); 
      periodicPanel.add(expirSelBtn);

      tabbedChoice.add("Periodic");
      cardPanel.add(PERIODIC_PANEL,periodicPanel);
    }
    cardLayout.first(cardPanel);
    tabbedChoice.addItemListener(this);

    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();
    Panel panel = new Panel(gridbag);

    constraints.fill       = GridBagConstraints.HORIZONTAL;
    constraints.gridwidth  = GridBagConstraints.REMAINDER; 
    constraints.gridheight = 1; 
    gridbag.setConstraints(tabbedChoice,constraints); 
    panel.add(tabbedChoice);
    constraints.fill       = GridBagConstraints.NONE;
    gridbag.setConstraints(cardPanel,constraints); 
    panel.add(cardPanel);

    SeparatorCanvas separator = new SeparatorCanvas();
    constraints.fill       = GridBagConstraints.HORIZONTAL; 
    constraints.gridheight = GridBagConstraints.RELATIVE; 
    gridbag.setConstraints(separator,constraints); 
    panel.add(separator);

    constraints.fill       = GridBagConstraints.NONE;
    constraints.gridheight = GridBagConstraints.REMAINDER;
    constraints.insets     = new Insets(10,7,10,7);
    gridbag.setConstraints(btnPanel,constraints); 
    panel.add(btnPanel);
    add(panel);

    pack();
    setResizable(false);
    //setVisible(true);
  } // end init

  public boolean getCanceled() {
    return canceled;
  } // end getCanceled

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

  public boolean getDeleted() {
    return deleted;
  } // end getDeleted

  public CalendarEntry getCalendarEntry() {
    return entry;
  } // end getCalendarEntry

  private void makeMsgBox(String str) {
    MsgBoxDlg msgBoxDlg = new MsgBoxDlg(parent,getTitle(),isModal(),str,
                                MsgBoxDlg.OK | MsgBoxDlg.ICONINFORMATION);
    if (autoCancel) {
      autoCancel = false;
      autoDisable = true;
      msgBoxDlg.addWindowListener(new MsgListener());
      msgBoxDlg.setAutoCancel(true);
    } // end if
  } // end makeMsgBox

  private boolean readResults() {
    // general settings
    Date date;
    ParsePosition pos;

    pos  = new ParsePosition(0);
    try { // this is a workaround, parse doesn't work properly
      date = dateTimeFormatter.parse(dateText.getText()+"."+
                                     startText.getText(),pos);
    } catch (NullPointerException e) { date = null; }
    if (date == null) {
      pos.setIndex(0);
      try { // this is a workaround, parse doesn't work properly
        date = dateFormatter.parse(dateText.getText(),pos);
      } catch (NullPointerException e) { date = null; }
      if (date == null) {
        tabbedChoice.select(0);
        dateText.setText(""); dateText.requestFocus();
        makeMsgBox("Please insert the date as 'dd.mm.yy'.");
      }
      else {
        tabbedChoice.select(0);
        startText.setText(""); startText.requestFocus();
        makeMsgBox("Please insert the start time as 'hh:mm'.");
      } // end if
      return false;
    } // end if
    entry.setStart(date);
    pos.setIndex(0);
    try {
      date = dateTimeFormatter.parse(dateText.getText()+"."+
                                     endText.getText(),pos);
    } catch (NullPointerException e) { date = null; }
    if (date == null) {
      tabbedChoice.select(0);
      endText.setText(""); endText.requestFocus();
      makeMsgBox("Please insert the end time as 'hh:mm'.");
      return false;
    } // end if
    // if the end time is'nt after the start time, then it must be on next day
    if (!date.after(entry.getStart())) {
      Calendar cal = CalendarCreator.get();
      cal.setTime(date);
      cal.add(Calendar.DATE,1);
      date = cal.getTime();
    } // end if
    entry.setEnd(date);
    entry.setDescription(descText.getText());
    entry.setLocation(locText.getText());
    if (alarmCheckbox.getState()) {
      boolean minutesSelected = (alarmChoice.getSelectedIndex() == 0);
      int uBound              =  minutesSelected ? 120 : 48;
      int value;

      try {
        value = Integer.parseInt(alarmText.getText());
      } catch (NumberFormatException e) { value = -1; }
      if ((value < 1) || (value > uBound)) {
        tabbedChoice.select(0);
        alarmText.setText(""); alarmText.requestFocus();
        makeMsgBox("Please insert a alert between 1 and 120 minutes\n"+
                   "or 1 and 48 hours.");
        return false;
      }
      else {
        Calendar cal = CalendarCreator.get();
        cal.setTime(entry.getStart());
        if (minutesSelected) cal.add(Calendar.MINUTE,-value);
          else cal.add(Calendar.HOUR,-value);
        entry.setAlertTime(cal.getTime());
      } // end if
    }
    else {
      entry.setAlertTime(new Date(CalendarEntry.INVALID_DATE));
    } // end if
    entry.setPrivate(privateCheckbox.getState());
    // persons settings
    entry.setPersons(personsList.getSelectedItems());
    // notes settings
    entry.setNotes(notesText.getText());
    // periodic settings
    if (periodicCheckbox.getState()) {
      int value;
      TextField tf;
      int uBound;
      String unit;

      if (daylyCheckbox.getState()) {
        entry.setPeriodic(CalendarEntry.EVERY_DAY);
        tf     = daylyText;
        uBound = 31;
        unit   = "days";
      }
      else if (weeklyCheckbox.getState()) {
        entry.setPeriodic(CalendarEntry.EVERY_WEEK);
        tf     = weeklyText;
        uBound = 52;
        unit   = "weeks";
      }
      else {
        if (periodicChoice.getSelectedIndex() == 0) // same day
          entry.setPeriodic(CalendarEntry.EVERY_MONTH_EXACT);
        else
          entry.setPeriodic(CalendarEntry.EVERY_MONTH_MATCH);
        // end if
        tf     = monthlyText;
        uBound = 12;
        unit   = "month";
      } // end if
      try {
        value = Integer.parseInt(tf.getText());
      } catch (NumberFormatException e) { value = -1; }
      if ((value < 1) || (value > uBound)) {
        tabbedChoice.select(3);
        tf.setText(""); tf.requestFocus();
        makeMsgBox("Please insert a value between 1 and "+uBound+
                   " "+unit+"\nfor the distance between two periodic dates.");
        return false;
      }
      else {
        entry.setDistance(value);
      } // end if
      if (expirCheckbox.getState()) {
        pos.setIndex(0);
        try { // this is a workaround, parse doesn't work properly
          date = dateTimeFormatter.parse(expirText.getText()+"."+
                                         endText.getText(),pos);
        } catch (NullPointerException e) { date = null; }
        if (date == null) {
          tabbedChoice.select(3);
          expirText.setText(""); expirText.requestFocus();
          makeMsgBox("Please insert the expiration date as 'dd.mm.yy'.");
          return false;
        }
        entry.setExpirTime(date);
      }
      else {
        entry.setExpirTime(new Date(CalendarEntry.INVALID_DATE));
      } // end if
    }
    else {
      entry.setPeriodic(CalendarEntry.NOT_PERIODICAL);
    } // end if

    return true;
  } // end readResults

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("OK")) {
      if (readResults()) {
        setVisible(false);
        dispose();
      } // end if
    }
    else if (cmd.equals("Delete")) {
      deleted = true;
      setVisible(false);
      dispose();
    }
    else if (cmd.equals("Select")) {
      Calendar calendar = CalendarCreator.get();
      if (autoCancel) { autoCancel = false; autoDisable = true; }
      DateChooserDlg dateDlg = new DateChooserDlg(parent,"Select Date",
                                                  isModal(),calendar,
                                                  CalendarCreator.getLocale());
      dateDlg.addWindowListener(new DateListener(dateDlg));
      dateDlg.setVisible(true);
    }
    else { // if cmd equals cancel
      canceled = true;
      setVisible(false);
      dispose();
    } // end if
  } // end actionPerformed


  public void itemStateChanged(ItemEvent e) {
    ItemSelectable item = e.getItemSelectable();
    if (item instanceof TabbedChoice) {
      int index = ((TabbedChoice)item).getSelectedIndex();
      switch (index) {
        case 0:
          //cardLayout.first(cardPanel);
          cardLayout.show(cardPanel,GENERAL_PANEL);
        break;
        case 1:
          //cardLayout.first(cardPanel);
          //cardLayout.next(cardPanel);
          cardLayout.show(cardPanel,PERSONS_PANEL);
        break;
        case 2:
          //cardLayout.first(cardPanel);
          //cardLayout.next(cardPanel);
          //cardLayout.next(cardPanel);
          cardLayout.show(cardPanel,NOTES_PANEL);
        break;
        default: // case 3
          //cardLayout.first(cardPanel);
          //cardLayout.next(cardPanel);
          //cardLayout.next(cardPanel);
          //cardLayout.next(cardPanel);
          cardLayout.show(cardPanel,PERIODIC_PANEL);
      } // end switch
    }
    else {
      updateEnabled(tabbedChoice.getSelectedIndex());
    } // end if
  } // end itemStateChanged

private void updateEnabled(int index) {
  // calculates and sets the enabled/disabled state for widgets on pane index
  boolean cantEditFull = ((mode == VIEW) || (mode == VIEW_AND_DELETE));
  boolean cantEditRest = cantEditFull || (mode == FIND_RESULT_NEW);

  switch (index) {
    case 0: {
      boolean alarmSelected = alarmCheckbox.getState();
    
      if (cantEditRest && dateText.isEditable()) {
        dateText.setEditable(false);
        dateSelBtn.setEnabled(false);
        startText.setEditable(false);
        endText.setEditable(false);
      } // end if
      if (cantEditFull && descText.isEditable()) {
        descText.setEditable(false);
        locText.setEditable(false);
        alarmCheckbox.setEnabled(false);
        privateCheckbox.setEnabled(false);
      } // end if
      alarmText.setEnabled(alarmSelected && !cantEditFull);
      alarmLabel.setEnabled(alarmSelected);
      alarmChoice.setEnabled(alarmSelected && !cantEditFull);
    } break;
    case 1:
      if (cantEditRest) personsList.setEnabled(false);
    break;
    case 2:
      if (cantEditFull) notesText.setEditable(false);
    break;
    default: { // case 3
      boolean periodicSelected = periodicCheckbox.getState() &&
                                 periodicCheckbox.isEnabled() &&
                                 (!cantEditRest);
      boolean daylySelected    = periodicSelected && daylyCheckbox.getState();
      boolean weeklySelected   = periodicSelected && weeklyCheckbox.getState();
      boolean monthlySelected  = periodicSelected && monthlyCheckbox.getState();
      boolean expirSelected    = periodicSelected && expirCheckbox.getState();      

      if (cantEditRest) periodicCheckbox.setEnabled(false);

      daylyCheckbox.setEnabled(periodicSelected);
      weeklyCheckbox.setEnabled(periodicSelected);
      monthlyCheckbox.setEnabled(periodicSelected);
  
      daylyText.setEnabled(daylySelected);
      daylyLabel.setEnabled(daylySelected);
      weeklyText.setEnabled(weeklySelected);
      weeklyLabel.setEnabled(weeklySelected);
      monthlyText.setEnabled(monthlySelected);
      periodicChoice.setEnabled(monthlySelected);
      monthlyLabel1.setEnabled(monthlySelected);
      monthlyLabel2.setEnabled(monthlySelected);

      expirCheckbox.setEnabled(periodicSelected);
      expirText.setEnabled(expirSelected);
      expirSelBtn.setEnabled(expirSelected);
    }
  } // end switch
} // end updateEnabled

} // end EditDateDlg

/* ******************************************************************
   end of file
*********************************************************************/
