/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac             

   class      : LoginDlg
   date       : 5.12.97
   description: a dialog that prompts user id and password
   modified   : 2.2.98 - auto cancel mode for applet operation added

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/


import java.awt.*; 
import java.awt.event.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class LoginDlg extends Dialog
  implements ActionListener,TextListener {

  // instance variable declarations
  private boolean canceled   = false;
  private boolean autoCancel = false;
  private Button okButton;
  private Button cancelButton;
  private TextField userText;
  private TextField passwdText;

  // inner classes
  class LoginDlgListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      setCanceled(true);
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        setCanceled(true);
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end LoginDlgListener

  // costructors
  public LoginDlg(Frame parent) {
    super(parent);
    init();
  } // end constructor LoginDlg

  public LoginDlg(Frame parent, boolean modal) {
    super(parent,modal);
    init();
  } // end constructor LoginDlg

  public LoginDlg(Frame parent, String title) {
    super(parent,title);
    init();
  } // end constructor LoginDlg

  public LoginDlg(Frame parent, String title, boolean modal) {
    super(parent,title,modal);
    init();
  } // end constructor LoginDlg

  // methods
  private void init() {
    addWindowListener(new LoginDlgListener());
    // this dialog box uses the gridbaglayout to provide a pretty good layout
    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();

    // create the widgets
    InsetsPanel pan = new InsetsPanel(new Insets(15,20,15,20));
    okButton = new Button("OK");
    okButton.setEnabled(false);
    okButton.addActionListener(this);
    cancelButton = new Button("Cancel");
    cancelButton.addActionListener(this);
    InsetsPanel buttonPanel = new InsetsPanel(new Insets(15,0,0,0));
    buttonPanel.setLayout(new GridLayout(1,2,10,0));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);

    userText        = new TextField("",15);
    userText.addTextListener(this);
    passwdText      = new TextField("",15);
    passwdText.setEchoChar('*');
    Panel editPanel = new Panel();
    editPanel.setLayout(new GridLayout(2,2));
    editPanel.add(new Label("User:"));
    editPanel.add(userText);
    editPanel.add(new Label("Password:"));
    editPanel.add(passwdText);

    pan.setLayout(gridbag); 

    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.CENTER; 
    constraints.weightx    = 1.0; 
    constraints.weighty    = 1.0; 
    constraints.gridwidth  = GridBagConstraints.REMAINDER; 
    constraints.gridheight = GridBagConstraints.RELATIVE; 
    gridbag.setConstraints(editPanel,constraints); 
    pan.add(editPanel); 

    constraints.ipadx      = 0; 
    constraints.ipady      = 0; 
    constraints.weightx    = 0.0; 
    constraints.weighty    = 0.0; 
    constraints.gridwidth  = 1; 
    constraints.gridheight = GridBagConstraints.REMAINDER; 

    gridbag.setConstraints(buttonPanel, constraints); 
    pan.add(buttonPanel); 

    add(pan);
    pack(); 
    //setVisible(true);
  } // end init

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("Cancel")) canceled = true;
    setVisible(false);
    dispose();
  } // end actionPerformed

  public void textValueChanged(TextEvent e) {
    okButton.setEnabled(userText.getText().length() >= 1);
  } // end textValueChanged

  public boolean getCanceled() {
    return canceled;
  } // end getCanceled

  public void setCanceled(boolean canceled) {
    this.canceled = canceled;
  } // end getCanceled

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

  public String getUser() {
    return userText.getText();
  } // end getUser

  public void setUser(String user) {
    userText.setText(user);
  } // end setUser

  public String getPasswd() {
    return passwdText.getText();
  } // end getPasswd

  public void setPasswd(String passwd) {
    passwdText.setText(passwd);
  } // end setPasswd

} // end LoginDlg

/* ******************************************************************
   end of file
*********************************************************************/
