MODULE Distributions; (* JT 30. 6.89 HM 17.6.92 MH 27.5.93 *)

IMPORT Math;

CONST
	m = 2147483647; a = 16807; q = m DIV a; r = m MOD a;

TYPE
	Distribution* = POINTER TO DistributionDesc;
	DistributionDesc* = RECORD
		z: LONGINT
	END;

	UniformDistribution* = POINTER TO UniformDistributionDesc;
	UniformDistributionDesc* = RECORD (DistributionDesc)
	END;

	ExpDistribution* = POINTER TO ExpDistributionDesc;
	ExpDistributionDesc* = RECORD (DistributionDesc)
		mean: REAL;
		u: UniformDistribution
	END;

	NormalDistribution* = POINTER TO NormalDistributionDesc;
	NormalDistributionDesc* = RECORD (DistributionDesc)
		mean: REAL;
		sDev: REAL;
		u: UniformDistribution
	END;

(* --- class Distribution *)
	
PROCEDURE (d: Distribution) Next* (): REAL;
BEGIN HALT(99)
END Next;

PROCEDURE (d: Distribution) SetSeed* (seed: LONGINT);
BEGIN d.z := seed
END SetSeed;

	
(* --- class UniformDistribution *)

PROCEDURE (d: UniformDistribution) Next* (): REAL;  (*according to M.Reiser*)
	VAR x: LONGINT;
BEGIN
	IF d.z = 0 THEN d.z := 1 END;
	x := a * (d.z MOD q) - r * (d.z DIV q);
	IF x > 0 THEN d.z := x ELSE d.z := x + m END;
	RETURN d.z / m
END Next;
	

(* --- class ExpDistribution *)

PROCEDURE (d: ExpDistribution) Next* (): REAL;
BEGIN
	RETURN -Math.ln(d.u.Next()) * d.mean
END Next;

PROCEDURE (d: ExpDistribution) Init* (mean: REAL);
BEGIN d.mean := mean; NEW(d.u)
END Init;


(* --- class NormalDistribution *)

PROCEDURE (d: NormalDistribution) Next*(): REAL;
	VAR v1, v2, s, x: REAL;
BEGIN
	REPEAT
		v1 := 2 * d.u.Next() - 1; v2 := 2 * d.u.Next() - 1;
		s := v1 * v1 + v2 * v2
	UNTIL s < 1;
	x := v1 * Math.sqrt((-2 * Math.ln(s)) / s);	(* << mh 27.5.93 *)
	RETURN d.mean + d.sDev * x
END Next;
	
PROCEDURE (d: NormalDistribution) Init* (m, sd: REAL);
BEGIN NEW(d.u); d.mean := m; d.sDev := sd
END Init;
	

END Distributions.
